@@ -20,7 +20,7 @@ except ImportError:
 
 from tempfile import NamedTemporaryFile, mkdtemp
 from osc.fetch import *
-from osc.core import get_buildinfo, store_read_apiurl, store_read_project, store_read_package, meta_exists, quote_plus, get_buildconfig, is_package_dir
+from osc.core import get_buildinfo, store_read_apiurl, store_read_project, store_read_package, meta_exists, quote_plus, get_buildconfig, is_package_dir, dgst
 from osc.core import get_binarylist, get_binary_file, run_external, raw_input
 from osc.util import rpmquery, debquery, archquery
 import osc.conf
@@ -200,7 +200,8 @@ class Pac:
         for i in ['binary', 'package',
                   'epoch', 'version', 'release', 'hdrmd5',
                   'project', 'repository',
-                  'preinstall', 'vminstall', 'noinstall', 'installonly', 'runscripts',
+                  'preinstall', 'vminstall', 'runscripts',
+                  'noinstall', 'installonly', 'notmeta',
                  ]:
             self.mp[i] = node.get(i)
 
@@ -229,7 +230,9 @@ class Pac:
 
         self.mp['apiurl'] = apiurl
 
-        if pacsuffix == 'deb':
+        if self.mp['name'].startswith('container:'):
+            canonname = self.mp['name'] + '.tar.xz'
+        elif pacsuffix == 'deb':
             canonname = debquery.DebQuery.filename(self.mp['name'], self.mp['epoch'], self.mp['version'], self.mp['release'], self.mp['arch'])
         elif pacsuffix == 'arch':
             canonname = archquery.ArchQuery.filename(self.mp['name'], self.mp['epoch'], self.mp['version'], self.mp['release'], self.mp['arch'])
@@ -341,6 +344,11 @@ def get_built_files(pacdir, buildtype):
                                     '-type', 'f'],
                                    stdout=subprocess.PIPE).stdout.read().strip()
         s_built = ''
+    elif buildtype == 'docker':
+        b_built = subprocess.Popen(['find', os.path.join(pacdir, 'DOCKER'),
+                                    '-type', 'f'],
+                                   stdout=subprocess.PIPE).stdout.read().strip()
+        s_built = ''
     elif buildtype == 'dsc' or buildtype == 'collax':
         b_built = subprocess.Popen(['find', os.path.join(pacdir, 'DEBS'),
                                     '-name', '*.deb'],
@@ -363,6 +371,11 @@ def get_built_files(pacdir, buildtype):
                                     '-name', '*.snap'],
                                    stdout=subprocess.PIPE).stdout.read().strip()
         s_built = ''
+    elif buildtype == 'appimage':
+        b_built = subprocess.Popen(['find', os.path.join(pacdir, 'OTHER'),
+                                    '-name', '*.AppImage'],
+                                   stdout=subprocess.PIPE).stdout.read().strip()
+        s_built = ''
     else:
         print('WARNING: Unknown package type \'%s\'.' % buildtype, file=sys.stderr)
         b_built = ''
@@ -501,11 +514,15 @@ def main(apiurl, opts, argv):
         build_type = 'arch'
     if os.path.basename(build_descr) == 'build.collax':
         build_type = 'collax'
+    if os.path.basename(build_descr) == 'appimage.yml':
+        build_type = 'appimage'
     if os.path.basename(build_descr) == 'snapcraft.yaml':
         build_type = 'snapcraft'
-    if build_type not in ['spec', 'dsc', 'kiwi', 'arch', 'collax', 'livebuild', 'snapcraft']:
+    if os.path.basename(build_descr) == 'Dockerfile':
+        build_type = 'docker'
+    if build_type not in ['spec', 'dsc', 'kiwi', 'arch', 'collax', 'livebuild', 'snapcraft', 'appimage', 'docker']:
         raise oscerr.WrongArgs(
-                'Unknown build type: \'%s\'. Build description should end in .spec, .dsc, .kiwi, .yaml or .livebuild.' \
+                'Unknown build type: \'%s\'. Build description should end in .spec, .dsc, .kiwi, or .livebuild. Or being named PKGBUILD, build.collax, appimage.yml, snapcraft.yaml or Dockerfile' \
                         % build_type)
     if not os.path.isfile(build_descr):
         raise oscerr.WrongArgs('Error: build description file named \'%s\' does not exist.' % build_descr)
@@ -587,6 +604,9 @@ def main(apiurl, opts, argv):
             pac = '_repository'
         else:
             pac = store_read_package(os.curdir)
+    if opts.multibuild_package:
+        buildargs.append('--buildflavor=%s' % opts.multibuild_package)
+        pac = pac + ":" + opts.multibuild_package
     if opts.shell:
         buildargs.append("--shell")
 
@@ -795,6 +815,9 @@ def main(apiurl, opts, argv):
     if bi.release:
         buildargs.append('--release=%s' % bi.release)
 
+    if opts.build_opt:
+        buildargs += opts.build_opt
+
     # real arch of this machine
     # vs.
     # arch we are supposed to build for
@@ -938,29 +961,41 @@ def main(apiurl, opts, argv):
         if old_pkg_dir != None:
             buildargs.append('--oldpackages=%s' % old_pkg_dir)
 
-    # Make packages from buildinfo available as repos for kiwi
-    if build_type == 'kiwi':
+    # Make packages from buildinfo available as repos for kiwi/docker
+    if build_type == 'kiwi' or build_type == 'docker':
         if os.path.exists('repos'):
             shutil.rmtree('repos')
+        if os.path.exists('containers'):
+            shutil.rmtree('containers')
         os.mkdir('repos')
         for i in bi.deps:
             if not i.extproject:
                 # remove
                 bi.deps.remove(i)
                 continue
+            if i.notmeta:
+                continue
             # project
             pdir = str(i.extproject).replace(':/', ':')
             # repo
             rdir = str(i.extrepository).replace(':/', ':')
             # arch
             adir = i.repoarch
-            # project/repo
-            prdir = "repos/"+pdir+"/"+rdir
-            # project/repo/arch
-            pradir = prdir+"/"+adir
             # source fullfilename
             sffn = i.fullfilename
             filename = sffn.split("/")[-1]
+            # project/repo
+            if i.name.startswith("container:"):
+                prdir = "containers/"+pdir+"/"+rdir
+                pradir = prdir
+                filename = filename[10:]
+                if build_type == 'kiwi':
+                    buildargs.append('--kiwi-parameter')
+                    buildargs.append('--set-container-derived-from=dir://./' + prdir + "/" + filename)
+            else:
+                prdir = "repos/"+pdir+"/"+rdir
+                # project/repo/arch
+                pradir = prdir+"/"+adir
             # target fullfilename
             tffn = pradir+"/"+filename
             if not os.path.exists(os.path.join(pradir)):
@@ -980,6 +1015,8 @@ def main(apiurl, opts, argv):
                             os.link(path + "/" + filename, tffn)
                         else:
                             os.symlink(path + "/" + filename, tffn)
+
+    if build_type == 'kiwi':
         # Is a obsrepositories tag used?
         try:
             tree = ET.parse(build_descr)
@@ -1009,7 +1046,7 @@ def main(apiurl, opts, argv):
                       buildargs.append('--kiwi-parameter')
                       buildargs.append('--add-repo')
                       buildargs.append('--kiwi-parameter')
-                      buildargs.append("repos/"+path)
+                      buildargs.append("dir://./repos/"+path)
                       buildargs.append('--kiwi-parameter')
                       buildargs.append('--add-repotype')
                       buildargs.append('--kiwi-parameter')
@@ -1027,7 +1064,7 @@ def main(apiurl, opts, argv):
                    buildargs.append('--kiwi-parameter')
                    buildargs.append('--add-repo')
                    buildargs.append('--kiwi-parameter')
-                   buildargs.append("repos/"+project+"/"+repo)
+                   buildargs.append("dir://./repos/"+project+"/"+repo)
                    buildargs.append('--kiwi-parameter')
                    buildargs.append('--add-repotype')
                    buildargs.append('--kiwi-parameter')
@@ -1055,7 +1092,10 @@ def main(apiurl, opts, argv):
     for i in bi.deps:
         if i.hdrmd5:
             from .util import packagequery
-            hdrmd5 = packagequery.PackageQuery.queryhdrmd5(i.fullfilename)
+            if i.name.startswith('container:'):
+                hdrmd5 = dgst(i.fullfilename)
+            else:
+                hdrmd5 = packagequery.PackageQuery.queryhdrmd5(i.fullfilename)
             if not hdrmd5:
                 print("Error: cannot get hdrmd5 for %s" % i.fullfilename)
                 sys.exit(1)
@@ -1065,7 +1105,7 @@ def main(apiurl, opts, argv):
 
     print('Writing build configuration')
 
-    if build_type == 'kiwi':
+    if build_type == 'kiwi' or build_type == 'docker':
         rpmlist = [ '%s %s\n' % (i.name, i.fullfilename) for i in bi.deps if not i.noinstall ]
     else:
         rpmlist = [ '%s %s\n' % (i.name, i.fullfilename) for i in bi.deps ]
@@ -1081,9 +1121,9 @@ def main(apiurl, opts, argv):
     rpmlist.append('preinstall: ' + ' '.join(bi.preinstall_list) + '\n')
     rpmlist.append('vminstall: ' + ' '.join(bi.vminstall_list) + '\n')
     rpmlist.append('runscripts: ' + ' '.join(bi.runscripts_list) + '\n')
-    if build_type != 'kiwi' and bi.noinstall_list:
+    if build_type != 'kiwi' and build_type != 'docker' and bi.noinstall_list:
         rpmlist.append('noinstall: ' + ' '.join(bi.noinstall_list) + '\n')
-    if build_type != 'kiwi' and bi.installonly_list:
+    if build_type != 'kiwi' and build_type != 'docker' and bi.installonly_list:
         rpmlist.append('installonly: ' + ' '.join(bi.installonly_list) + '\n')
 
     rpmlist_file = NamedTemporaryFile(prefix='rpmlist.')
